import scanpy as sc
import numpy as np
import pandas as pd
from ..utils.registry import register_function

sc_color=[
 '#1F577B', '#A56BA7', '#E0A7C8', '#E069A6', '#941456', 
 '#FCBC10', '#EF7B77', '#279AD7','#F0EEF0',
 '#EAEFC5', '#7CBB5F','#368650','#A499CC','#5E4D9A',
 '#78C2ED','#866017', '#9F987F','#E0DFED',
 '#01A0A7', '#75C8CC', '#F0D7BC', '#D5B26C', '#D5DA48',
 '#B6B812', '#9DC3C3', '#A89C92', '#FEE00C', '#FEF2A1']

red_color=['#F0C3C3','#E07370','#CB3E35','#A22E2A','#5A1713',
           '#D3396D','#8B0000', '#A52A2A', '#CD5C5C', '#DC143C' ]

green_color=['#91C79D','#8FC155','#56AB56','#2D5C33','#BBCD91',
             '#6E944A','#A5C953','#3B4A25','#010000']

orange_color=['#EFBD49','#D48F3E','#AC8A3E','#7D7237','#745228',
              '#E1C085','#CEBC49','#EBE3A1','#6C6331','#8C9A48','#D7DE61']

blue_color=['#1F577B', '#279AD7', '#78C2ED', '#01A0A7', '#75C8CC', '#9DC3C3',
            '#3E8CB1', '#52B3AD', '#265B58', '#5860A7', '#312C6C', '#4CC9F0']

purple_color=['#823d86','#825b94','#bb98c6','#c69bc6','#a69ac9',
              '#c5a6cc','#caadc4','#d1c3d4']

#more beautiful colors
# 28-color palettes with distinct neighboring colors
palette_28 = sc_color[:28]
# 56-color palette with clear transitions

# 112-color palette with distinct transitions
cet_g_bw = [
 '#d60000', '#8c3bff', '#018700', '#00acc6', '#97ff00', '#ff7ed1', '#6b004f', '#ffa52f', '#00009c', '#857067',
 '#004942', '#4f2a00', '#00fdcf', '#bcb6ff', '#95b379', '#bf03b8', '#2466a1', '#280041', '#dbb3af', '#fdf490',
 '#4f445b', '#a37c00', '#ff7066', '#3f806e', '#82000c', '#a37bb3', '#344d00', '#9ae4ff', '#eb0077', '#2d000a',
 '#5d90ff', '#00c61f', '#5701aa', '#001d00', '#9a4600', '#959ea5', '#9a425b', '#001f31', '#c8c300', '#ffcfff',
 '#00bd9a', '#3615ff', '#2d2424', '#df57ff', '#bde6bf', '#7e4497', '#524f3b', '#d86600', '#647438', '#c17287',
 '#6e7489', '#809c03', '#bd8a64', '#623338', '#cacdda', '#6beb82', '#213f69', '#a17eff', '#fd03ca', '#75bcfd',
 '#d8c382', '#cda3cd', '#6d4f00', '#006974', '#469e5d', '#93c6bf', '#f9ff00', '#bf5444', '#00643b', '#5b4fa8',
 '#521f64', '#4f5eff', '#7e8e77', '#b808f9', '#8a91c3', '#b30034', '#87607e', '#9e0075', '#ffddc3', '#500800',
 '#1a0800', '#4b89b5', '#00dfdf', '#c8fff9', '#2f3415', '#ff2646', '#ff97aa', '#03001a', '#c860b1', '#c3a136',
 '#7c4f3a', '#f99e77', '#566464', '#d193ff', '#2d1f69', '#411a34', '#af9397', '#629e99', '#bcdd7b', '#ff5d93',
 '#0f2823', '#b8bdac', '#743b64', '#0f000c', '#7e6ebc', '#9e6b3b', '#ff4600', '#7e0087', '#ffcd3d', '#2f3b42',
 '#fda5ff', '#89013d', '#752b01', '#0a8995', '#050052', '#8ed631', '#52c372', '#465970', '#570121', '#a52101',
 '#90934b', '#00421d', '#8000d1', '#2f263f', '#bf3883', '#f4ffd4', '#00d3ff', '#6900f7', '#9cbad1', '#79d8aa',
 '#69565d', '#006905', '#36369c', '#018246', '#441d18', '#07a5ef', '#ff802f', '#a754b8', '#675982', '#72ffff',
 '#d88701', '#bad3ff', '#8e362f', '#a7a080', '#007ce2', '#8e7e8e', '#994487', '#00f034', '#aeaac8', '#a06062',
 '#4b3a77', '#6b8282', '#f0dde6', '#ffbad3', '#38a523', '#b3ffa8', '#0c1107', '#d6526e', '#959efd', '#7c7e00',
 '#759eb8', '#db877e', '#111318', '#d482d4', '#9e00bf', '#dbefff', '#8eaa9a', '#706442', '#493b3d', '#084d5e',
 '#9cb844', '#d8ddd4', '#caff6b', '#b364eb', '#465d33', '#009e7c', '#c14100', '#4fbcba', '#d88ab1', '#5b72b5',
 '#4b4101', '#95825d', '#49748a', '#ff72ff', '#82691c', '#dbcfff', '#7e6bfd', '#627560', '#ffc191', '#595d00',
 '#e408e6', '#b8b1b6', '#d32d41', '#314236', '#d8a362', '#5b8a33', '#2f1f00', '#97e6d6', '#2a6256', '#cd724d',
 '#5d3d28', '#0059d8', '#ac93d6', '#6b1d93', '#b3015d', '#410046', '#9cffcf', '#e4489c', '#e2e246', '#dbe2a5',
 '#002859', '#aa5b82', '#0000db', '#4b4d50', '#dabfd4', '#004d99', '#87649e', '#691d1c', '#8e52c4', '#b8dadf',
 '#ddb3fd', '#7b4854', '#4b7200', '#440077', '#b15e00', '#91d185', '#54334b', '#69af85', '#aa93af', '#e65442',
 '#8e8c89', '#70ac50', '#aa7c74', '#00343b', '#240f13', '#e6af00', '#79ccdb', '#18133a', '#9c5238', '#ba7b31',
 '#b6ca93', '#310800', '#a39505', '#00daba', '#74a0dd', '#623b72', '#ffda8e', '#77b800', '#3f2f1c', '#578759',
 '#2d0021', '#f4a1d4', '#da00aa', '#752849', '#bce400', '#c3c15d'
]

palette_112 = cet_g_bw[:112]
palette_56 = cet_g_bw[:56]

# Vibrant palette with clear distinctions (24 colors)
vibrant_palette = [
    '#FF0000', '#00FF00', '#0000FF', '#FFFF00', '#FF00FF', '#00FFFF', 
    '#FF8000', '#FF0080', '#80FF00', '#00FF80', '#8000FF', '#0080FF',
    '#FF3333', '#33FF33', '#3333FF', '#FFFF33', '#FF33FF', '#33FFFF',
    '#FF9933', '#FF3399', '#99FF33', '#33FF99', '#9933FF', '#3399FF'
]

# Earth tones palette (24 colors)
earth_palette = [
    '#8B4513', '#DAA520', '#556B2F', '#2F4F4F', '#8B008B', '#4682B4',
    '#CD853F', '#BDB76B', '#6B8E23', '#4F666A', '#800080', '#4169E1',
    '#D2691E', '#F0E68C', '#9ACD32', '#5F9EA0', '#9932CC', '#1E90FF',
    '#A0522D', '#EEE8AA', '#698B22', '#008B8B', '#9400D3', '#00BFFF'
]

# Pastel palette (24 colors)
pastel_palette = [
    '#FFB3BA', '#BAFFC9', '#BAE1FF', '#FFFFBA', '#FFDFBA', '#E0BBE4',
    '#957DAD', '#D291BC', '#FEC8D8', '#FFDFD3', '#B5EAD7', '#C7CEEA',
    '#FFB997', '#F5B0CB', '#D4F0F0', '#FFF5BA', '#A8E6CF', '#DBB4D8',
    '#FFD3B5', '#D4F0F0', '#B5EAD7', '#E2F0CB', '#C7CEEA', '#FFDAC1'
]

def colormaps_palette(map_name:str)->list:
    r"""Returns a colormap palette.

    colormap's name could be found in `https://pratiman-91.github.io/colormaps/`
    
    Arguments:
        map_name: Name of the colormap
        
    Returns:
        List of hex color codes for the colormap
    """
    import colormaps
    import matplotlib.colors as mcolors
    cmap = getattr(colormaps, map_name)
    return [mcolors.to_hex(i) for i in cmap.colors]

def optim_palette(adata,basis,colors,palette=None,**kwargs):
    """
    Optimized palette for plotting

    Arguments:
        adata: AnnData object
        basis: basis for plotting, which is the key of adata.obsm
        colors: key of adata.obs for color
        palette: palette for plotting
        kwargs: kwargs for spaco.colorize

    Returns:
        palette_spaco: palette for plotting
    
    """

    try:
        import spaco
        #print('mofax have been install version:',mfx.__version__)
    except ImportError:
        raise ImportError(
            'Please install the spaco: `pip install spaco-release`.'
        )
    
    adata.obs[colors]=adata.obs[colors].astype('category')

    if (adata.uns[f'{colors}_colors'] is None) and (palette is None):
        if len(adata.obs[colors].cat.categories)>28:
            palette_t=sc.pl.palettes.default_102[:len(list(set(adata.obs[colors].tolist())))]
        else:
            palette_t=sc.pl.palettes.zeileis_28[:len(list(set(adata.obs[colors].tolist())))]
        #palette_t=ditto_color[:len(list(set(adata.obs[colors].tolist())))]
    elif palette!= None:
        palette_t=palette[:len(list(set(adata.obs[colors].tolist())))]
    else:
        palette_t=adata.uns[f'{colors}_colors']
        
    import spaco
    color_mapping = spaco.colorize(
        cell_coordinates=adata.obsm[basis],
        cell_labels=adata.obs[colors],
        colorblind_type='none',
        palette=palette_t,
        **kwargs
    )
    # Order colors by categories in adata
    color_mapping = {k: color_mapping[k] for k in adata.obs[colors].cat.categories}
    palette_spaco = list(color_mapping.values())
    return palette_spaco

def Forbidden_Cmap(color_range,return_cmap=False):
    r"""
    Create a colormap or color list from the Forbidden City color palette.
    
    Arguments:
        color_range: list or sequence
            Range of color indices to select from the Forbidden City palette
        return_cmap: bool, optional (default=False)
            Whether to return a matplotlib colormap object or HTML color codes
    
    Returns:
        If return_cmap is True, returns matplotlib.colors.ListedColormap
        If return_cmap is False, returns list of HTML color codes
    """
    from matplotlib.colors import ListedColormap
    import numpy as np
    colors = [Forbiddencity[str(color)]['color_rgb'] for color in color_range]
    # Create a ListedColormap
    if return_cmap:
        cmap = ListedColormap(np.array(colors) / 255.0)
        return cmap
    else:
        return [Forbiddencity[str(color)]['color_html'] for color in color_range]
    

def get_forbidden():
    r"""
    Get the complete Forbidden City color palette dictionary.
    
    Returns:
        dict: Dictionary containing traditional Chinese color names and their RGB/HTML values
        Each color entry contains: num, name, name_en, color_rgb, color_html
    """
    Forbiddencity={'1': {'num': '1',
    'name': '人籁',
    'name_en': 'Anthropophony',
    'color_rgb': (156, 188, 28),
    'color_html': '#9cbc1c'},
    '2': {'num': '2',
    'name': '青粲',
    'name_en': 'Verdigris',
    'color_rgb': (196, 220, 76),
    'color_html': '#c4dc4c'},
    '3': {'num': '3',
    'name': '翠缥',
    'name_en': 'Jadegreen',
    'color_rgb': (180, 212, 52),
    'color_html': '#b4d434'},
    '4': {'num': '4',
    'name': '水龙吟',
    'name_en': 'Waterdragonchant',
    'color_rgb': (132, 164, 44),
    'color_html': '#84a42c'},
    '5': {'num': '5',
    'name': '碧山',
    'name_en': 'Jadegreen',
    'color_rgb': (116, 148, 76),
    'color_html': '#74944c'},
    '6': {'num': '6',
    'name': '石发',
    'name_en': 'Stonehair',
    'color_rgb': (108, 140, 84),
    'color_html': '#6c8c54'},
    '7': {'num': '7',
    'name': '菉竹',
    'name_en': 'Verdantflute',
    'color_rgb': (108, 140, 108),
    'color_html': '#6c8c6c'},
    '8': {'num': '8',
    'name': '庭芜绿',
    'name_en': 'Courtyardverdure',
    'color_rgb': (108, 148, 92),
    'color_html': '#6c945c'},
    '9': {'num': '9',
    'name': '葱倩',
    'name_en': 'Verdantgrace',
    'color_rgb': (108, 132, 84),
    'color_html': '#6c8454'},
    '10': {'num': '10',
    'name': '漆姑',
    'name_en': 'Lacquermaid',
    'color_rgb': (92, 132, 84),
    'color_html': '#5c8454'},
    '11': {'num': '11',
    'name': '翠微',
    'name_en': 'Jadegreen',
    'color_rgb': (76, 132, 68),
    'color_html': '#4c8444'},
    '12': {'num': '12',
    'name': '芰荷',
    'name_en': 'Waterbloom',
    'color_rgb': (76, 124, 76),
    'color_html': '#4c7c4c'},
    '13': {'num': '13',
    'name': '青青',
    'name_en': 'Verdant',
    'color_rgb': (76, 108, 76),
    'color_html': '#4c6c4c'},
    '14': {'num': '14',
    'name': '翠虬',
    'name_en': 'JadeDragon',
    'color_rgb': (68, 108, 52),
    'color_html': '#446c34'},
    '15': {'num': '15',
    'name': '官绿',
    'name_en': 'MandarinJade',
    'color_rgb': (44, 108, 60),
    'color_html': '#2c6c3c'},
    '16': {'num': '16',
    'name': '油绿',
    'name_en': 'Olivine',
    'color_rgb': (92, 116, 92),
    'color_html': '#5c745c'},
    '17': {'num': '17',
    'name': '莓莓',
    'name_en': 'Strawberrymore',
    'color_rgb': (76, 100, 76),
    'color_html': '#4c644c'},
    '18': {'num': '18',
    'name': '螺青',
    'name_en': 'Spiralgreen',
    'color_rgb': (60, 84, 60),
    'color_html': '#3c543c'},
    '19': {'num': '19',
    'name': '春辰',
    'name_en': 'Springmorn',
    'color_rgb': (172, 188, 124),
    'color_html': '#acbc7c'},
    '20': {'num': '20',
    'name': '麴尘',
    'name_en': 'Chlorella',
    'color_rgb': (196, 212, 156),
    'color_html': '#c4d49c'},
    '21': {'num': '21',
    'name': '欧碧',
    'name_en': 'Aubergine',
    'color_rgb': (196, 212, 148),
    'color_html': '#c4d494'},
    '22': {'num': '22',
    'name': '苍葭',
    'name_en': 'Mistreed',
    'color_rgb': (172, 188, 140),
    'color_html': '#acbc8c'},
    '23': {'num': '23',
    'name': '兰苕',
    'name_en': 'Lantiau',
    'color_rgb': (172, 180, 140),
    'color_html': '#acb48c'},
    '24': {'num': '24',
    'name': '青玉案',
    'name_en': 'Jadequery',
    'color_rgb': (172, 180, 148),
    'color_html': '#acb494'},
    '25': {'num': '25',
    'name': '碧滋',
    'name_en': 'Berylize',
    'color_rgb': (148, 164, 124),
    'color_html': '#94a47c'},
    '26': {'num': '26',
    'name': '瓷秘',
    'name_en': 'Porceligence',
    'color_rgb': (188, 196, 148),
    'color_html': '#bcc494'},
    '27': {'num': '27',
    'name': '筠雾',
    'name_en': 'Bamboomist',
    'color_rgb': (212, 212, 172),
    'color_html': '#d4d4ac'},
    '28': {'num': '28',
    'name': '竹香子',
    'name_en': 'Bambooscent',
    'color_rgb': (188, 188, 156),
    'color_html': '#bcbc9c'},
    '29': {'num': '29',
    'name': '鸣珂',
    'name_en': 'Chimecarol',
    'color_rgb': (180, 180, 156),
    'color_html': '#b4b49c'},
    '30': {'num': '30',
    'name': '琬琰',
    'name_en': 'Jadestone',
    'color_rgb': (172, 172, 132),
    'color_html': '#acac84'},
    '31': {'num': '31',
    'name': '出岫',
    'name_en': 'Mistbirth',
    'color_rgb': (172, 164, 116),
    'color_html': '#aca474'},
    '32': {'num': '32',
    'name': '王刍',
    'name_en': 'Regalherb',
    'color_rgb': (172, 156, 116),
    'color_html': '#ac9c74'},
    '33': {'num': '33',
    'name': '春碧',
    'name_en': 'Springjade',
    'color_rgb': (156, 156, 132),
    'color_html': '#9c9c84'},
    '34': {'num': '34',
    'name': '执大象',
    'name_en': 'Elephantance',
    'color_rgb': (148, 148, 116),
    'color_html': '#949474'},
    '35': {'num': '35',
    'name': '青圭',
    'name_en': 'Azureadorn',
    'color_rgb': (148, 148, 92),
    'color_html': '#94945c'},
    '36': {'num': '36',
    'name': '绿沈',
    'name_en': 'Verdigris',
    'color_rgb': (148, 140, 76),
    'color_html': '#948c4c'},
    '37': {'num': '37',
    'name': '风入松',
    'name_en': 'Windthread',
    'color_rgb': (132, 140, 76),
    'color_html': '#848c4c'},
    '38': {'num': '38',
    'name': '荩篋',
    'name_en': 'Jinquet',
    'color_rgb': (132, 124, 84),
    'color_html': '#847c54'},
    '39': {'num': '39',
    'name': '绞衣',
    'name_en': 'Garmentwring',
    'color_rgb': (124, 116, 76),
    'color_html': '#7c744c'},
    '40': {'num': '40',
    'name': '素綦',
    'name_en': 'Sujy',
    'color_rgb': (92, 84, 52),
    'color_html': '#5c5434'},
    '41': {'num': '41',
    'name': '苍筤',
    'name_en': 'Canglang',
    'color_rgb': (156, 188, 172),
    'color_html': '#9cbcac'},
    '42': {'num': '42',
    'name': '天缥',
    'name_en': 'CelestialMist',
    'color_rgb': (212, 236, 228),
    'color_html': '#d4ece4'},
    '43': {'num': '43',
    'name': '卵色',
    'name_en': 'Ovatint',
    'color_rgb': (212, 228, 212),
    'color_html': '#d4e4d4'},
    '44': {'num': '44',
    'name': '沧浪',
    'name_en': 'Azurewave',
    'color_rgb': (180, 212, 204),
    'color_html': '#b4d4cc'},
    '45': {'num': '45',
    'name': '葭菼',
    'name_en': 'Reedwhisper',
    'color_rgb': (204, 212, 196),
    'color_html': '#ccd4c4'},
    '46': {'num': '46',
    'name': '山岚',
    'name_en': 'Mountainmist',
    'color_rgb': (188, 212, 188),
    'color_html': '#bcd4bc'},
    '47': {'num': '47',
    'name': '冰台',
    'name_en': 'Frostpod',
    'color_rgb': (188, 204, 180),
    'color_html': '#bcccb4'},
    '48': {'num': '48',
    'name': '青古',
    'name_en': 'Verdantique',
    'color_rgb': (180, 188, 172),
    'color_html': '#b4bcac'},
    '49': {'num': '49',
    'name': '醽醁',
    'name_en': 'Elixerian',
    'color_rgb': (164, 188, 180),
    'color_html': '#a4bcb4'},
    '50': {'num': '50',
    'name': '渌波',
    'name_en': 'Verdantwave',
    'color_rgb': (156, 180, 148),
    'color_html': '#9cb494'},
    '51': {'num': '51',
    'name': '青楸',
    'name_en': 'Azureoak',
    'color_rgb': (132, 164, 132),
    'color_html': '#84a484'},
    '52': {'num': '52',
    'name': '缥碧',
    'name_en': 'Azureglow',
    'color_rgb': (132, 164, 148),
    'color_html': '#84a494'},
    '53': {'num': '53',
    'name': '翠涛',
    'name_en': 'Jadewave',
    'color_rgb': (132, 156, 140),
    'color_html': '#849c8c'},
    '54': {'num': '54',
    'name': '青梅',
    'name_en': 'Greenplum',
    'color_rgb': (116, 140, 116),
    'color_html': '#748c74'},
    '55': {'num': '55',
    'name': '雀梅',
    'name_en': 'Sparrowplum',
    'color_rgb': (124, 140, 108),
    'color_html': '#7c8c6c'},
    '56': {'num': '56',
    'name': '苔古',
    'name_en': 'Mossantique',
    'color_rgb': (124, 132, 108),
    'color_html': '#7c846c'},
    '57': {'num': '57',
    'name': '蕉月',
    'name_en': 'Banamoon',
    'color_rgb': (132, 148, 140),
    'color_html': '#84948c'},
    '58': {'num': '58',
    'name': '干山翠',
    'name_en': 'Dryverdant',
    'color_rgb': (108, 124, 116),
    'color_html': '#6c7c74'},
    '59': {'num': '59',
    'name': '翕赩',
    'name_en': 'Flamewisp',
    'color_rgb': (92, 116, 108),
    'color_html': '#5c746c'},
    '60': {'num': '60',
    'name': '结绿',
    'name_en': 'Jadeknot',
    'color_rgb': (84, 92, 76),
    'color_html': '#545c4c'},
    '61': {'num': '61',
    'name': '绿云',
    'name_en': 'EmeraldCloud',
    'color_rgb': (68, 76, 60),
    'color_html': '#444c3c'},
    '62': {'num': '62',
    'name': '丹罽',
    'name_en': 'Crimsonweave',
    'color_rgb': (228, 4, 20),
    'color_html': '#e40414'},
    '63': {'num': '63',
    'name': '黄丹',
    'name_en': 'Aurelium',
    'color_rgb': (236, 84, 20),
    'color_html': '#ec5414'},
    '64': {'num': '64',
    'name': '檎丹',
    'name_en': 'Quindan',
    'color_rgb': (236, 76, 44),
    'color_html': '#ec4c2c'},
    '65': {'num': '65',
    'name': '银朱',
    'name_en': 'Vermilion',
    'color_rgb': (212, 44, 36),
    'color_html': '#d42c24'},
    '66': {'num': '66',
    'name': '洛神珠',
    'name_en': 'Lushenzhu',
    'color_rgb': (212, 60, 28),
    'color_html': '#d43c1c'},
    '67': {'num': '67',
    'name': '珊瑚赫',
    'name_en': 'Coralheart',
    'color_rgb': (196, 44, 28),
    'color_html': '#c42c1c'},
    '68': {'num': '68',
    'name': '朱孔阳',
    'name_en': 'Zhukeyang',
    'color_rgb': (188, 28, 52),
    'color_html': '#bc1c34'},
    '69': {'num': '69',
    'name': '丹雘',
    'name_en': 'Crimsonhue',
    'color_rgb': (204, 20, 28),
    'color_html': '#cc141c'},
    '70': {'num': '70',
    'name': '水华朱',
    'name_en': 'Crimsonbloom',
    'color_rgb': (164, 36, 36),
    'color_html': '#a42424'},
    '71': {'num': '71',
    'name': '胭脂虫',
    'name_en': 'Cochineal',
    'color_rgb': (172, 28, 36),
    'color_html': '#ac1c24'},
    '72': {'num': '72',
    'name': '朱樱',
    'name_en': 'Crimsonpearl',
    'color_rgb': (140, 28, 36),
    'color_html': '#8c1c24'},
    '73': {'num': '73',
    'name': '大繎',
    'name_en': 'Grandweave',
    'color_rgb': (132, 36, 36),
    'color_html': '#842424'},
    '74': {'num': '74',
    'name': '顺圣',
    'name_en': 'Flowbless',
    'color_rgb': (124, 28, 28),
    'color_html': '#7c1c1c'},
    '75': {'num': '75',
    'name': '爵头',
    'name_en': 'Crownhead',
    'color_rgb': (100, 20, 20),
    'color_html': '#641414'},
    '76': {'num': '76',
    'name': '麒麟竭',
    'name_en': 'Flameheart',
    'color_rgb': (76, 28, 28),
    'color_html': '#4c1c1c'},
    '77': {'num': '77',
    'name': '苕荣',
    'name_en': 'Vermillionia',
    'color_rgb': (236, 108, 60),
    'color_html': '#ec6c3c'},
    '78': {'num': '78',
    'name': '扶光',
    'name_en': 'Luminhold',
    'color_rgb': (244, 196, 164),
    'color_html': '#f4c4a4'},
    '79': {'num': '79',
    'name': '十样锦',
    'name_en': 'Tenbrocade',
    'color_rgb': (252, 196, 180),
    'color_html': '#fcc4b4'},
    '80': {'num': '80',
    'name': '海天霞',
    'name_en': 'Seaskyflush',
    'color_rgb': (244, 164, 148),
    'color_html': '#f4a494'},
    '81': {'num': '81',
    'name': '驿刚',
    'name_en': 'Eleganceport',
    'color_rgb': (244, 180, 132),
    'color_html': '#f4b484'},
    '82': {'num': '82',
    'name': '朱颜酡',
    'name_en': 'Rosygloam',
    'color_rgb': (244, 156, 116),
    'color_html': '#f49c74'},
    '83': {'num': '83',
    'name': '赦霞',
    'name_en': 'Pardonlight',
    'color_rgb': (244, 140, 100),
    'color_html': '#f48c64'},
    '84': {'num': '84',
    'name': '赦尾',
    'name_en': 'Pardontrail',
    'color_rgb': (236, 132, 92),
    'color_html': '#ec845c'},
    '85': {'num': '85',
    'name': '缙云',
    'name_en': 'Jinyun',
    'color_rgb': (236, 124, 92),
    'color_html': '#ec7c5c'},
    '86': {'num': '86',
    'name': '小红',
    'name_en': 'Crimsonette',
    'color_rgb': (228, 116, 100),
    'color_html': '#e47464'},
    '87': {'num': '87',
    'name': '琼琚',
    'name_en': 'Jadepearl',
    'color_rgb': (212, 124, 100),
    'color_html': '#d47c64'},
    '88': {'num': '88',
    'name': '岱赭',
    'name_en': 'Daizhuo',
    'color_rgb': (220, 108, 76),
    'color_html': '#dc6c4c'},
    '89': {'num': '89',
    'name': '朱柿',
    'name_en': 'Crimsonpersimmon',
    'color_rgb': (236, 108, 68),
    'color_html': '#ec6c44'},
    '90': {'num': '90',
    'name': '艳炽',
    'name_en': 'Resplendence',
    'color_rgb': (204, 84, 60),
    'color_html': '#cc543c'},
    '91': {'num': '91',
    'name': '鹤顶红',
    'name_en': 'Crimsoncrest',
    'color_rgb': (212, 68, 52),
    'color_html': '#d44434'},
    '92': {'num': '92',
    'name': '赤缇',
    'name_en': 'Crimsonthread',
    'color_rgb': (188, 92, 76),
    'color_html': '#bc5c4c'},
    '93': {'num': '93',
    'name': '纁黄',
    'name_en': 'Amaranthine',
    'color_rgb': (188, 84, 68),
    'color_html': '#bc5444'},
    '94': {'num': '94',
    'name': '棠梨',
    'name_en': 'Pearplum',
    'color_rgb': (180, 92, 68),
    'color_html': '#b45c44'},
    '95': {'num': '95',
    'name': '朱殷',
    'name_en': 'Crimsoness',
    'color_rgb': (188, 60, 36),
    'color_html': '#bc3c24'},
    '96': {'num': '96',
    'name': '石榴裙',
    'name_en': 'PomegranateGown',
    'color_rgb': (180, 60, 44),
    'color_html': '#b43c2c'},
    '97': {'num': '97',
    'name': '朱草',
    'name_en': 'Crimsonwort',
    'color_rgb': (164, 68, 52),
    'color_html': '#a44434'},
    '98': {'num': '98',
    'name': '赤灵',
    'name_en': 'CrimsonSoul',
    'color_rgb': (148, 68, 36),
    'color_html': '#944424'},
    '99': {'num': '99',
    'name': '佛赤',
    'name_en': 'Buddharub',
    'color_rgb': (140, 60, 44),
    'color_html': '#8c3c2c'},
    '100': {'num': '100',
    'name': '綪茷',
    'name_en': 'Crimsonweave',
    'color_rgb': (156, 44, 36),
    'color_html': '#9c2c24'},
    '101': {'num': '101',
    'name': '朱湛',
    'name_en': 'Crimsondeep',
    'color_rgb': (148, 52, 44),
    'color_html': '#94342c'},
    '102': {'num': '102',
    'name': '丹秫',
    'name_en': 'Crimsongrain',
    'color_rgb': (132, 52, 36),
    'color_html': '#843424'},
    '103': {'num': '103',
    'name': '木兰',
    'name_en': 'Magnoliahaven',
    'color_rgb': (100, 44, 28),
    'color_html': '#642c1c'},
    '104': {'num': '104',
    'name': '杨妃',
    'name_en': 'Yangfey',
    'color_rgb': (244, 148, 164),
    'color_html': '#f494a4'},
    '105': {'num': '105',
    'name': '盈盈',
    'name_en': 'Luminous',
    'color_rgb': (252, 212, 228),
    'color_html': '#fcd4e4'},
    '106': {'num': '106',
    'name': '银红',
    'name_en': 'Silverrose',
    'color_rgb': (228, 204, 212),
    'color_html': '#e4ccd4'},
    '107': {'num': '107',
    'name': '粉米',
    'name_en': 'Petalmeal',
    'color_rgb': (236, 196, 204),
    'color_html': '#ecc4cc'},
    '108': {'num': '108',
    'name': '桃夭',
    'name_en': 'Peachbloom',
    'color_rgb': (244, 188, 204),
    'color_html': '#f4bccc'},
    '109': {'num': '109',
    'name': '水红',
    'name_en': 'Crimsonwater',
    'color_rgb': (236, 180, 196),
    'color_html': '#ecb4c4'},
    '110': {'num': '110',
    'name': '夕岚',
    'name_en': 'EveningMist',
    'color_rgb': (228, 172, 188),
    'color_html': '#e4acbc'},
    '111': {'num': '111',
    'name': '彤管',
    'name_en': 'Crimsonquill',
    'color_rgb': (228, 164, 172),
    'color_html': '#e4a4ac'},
    '112': {'num': '112',
    'name': '咸池',
    'name_en': 'Xianchi',
    'color_rgb': (220, 172, 172),
    'color_html': '#dcacac'},
    '113': {'num': '113',
    'name': '莲红',
    'name_en': 'Lotuscrimson',
    'color_rgb': (220, 164, 180),
    'color_html': '#dca4b4'},
    '114': {'num': '114',
    'name': '雌霓',
    'name_en': 'Clenith',
    'color_rgb': (204, 148, 156),
    'color_html': '#cc949c'},
    '115': {'num': '115',
    'name': '縓缘',
    'name_en': 'Crimsonedge',
    'color_rgb': (204, 140, 148),
    'color_html': '#cc8c94'},
    '116': {'num': '116',
    'name': '长春',
    'name_en': 'EternalSpring',
    'color_rgb': (220, 108, 132),
    'color_html': '#dc6c84'},
    '117': {'num': '117',
    'name': '渥赭',
    'name_en': 'Crimsoned',
    'color_rgb': (220, 108, 124),
    'color_html': '#dc6c7c'},
    '118': {'num': '118',
    'name': '红䵂',
    'name_en': 'Crimsonwhisper',
    'color_rgb': (204, 116, 116),
    'color_html': '#cc7474'},
    '119': {'num': '119',
    'name': '紫梅',
    'name_en': 'Purpleplum',
    'color_rgb': (188, 124, 140),
    'color_html': '#bc7c8c'},
    '120': {'num': '120',
    'name': '绛纱',
    'name_en': 'Crimsonveil',
    'color_rgb': (180, 116, 116),
    'color_html': '#b47474'},
    '121': {'num': '121',
    'name': '茹藘',
    'name_en': 'Rulun',
    'color_rgb': (164, 92, 100),
    'color_html': '#a45c64'},
    '122': {'num': '122',
    'name': '美人祭',
    'name_en': 'Bellefest',
    'color_rgb': (196, 92, 108),
    'color_html': '#c45c6c'},
    '123': {'num': '123',
    'name': '唇脂',
    'name_en': 'Lipblossom',
    'color_rgb': (196, 84, 100),
    'color_html': '#c45464'},
    '124': {'num': '124',
    'name': '牙绯',
    'name_en': 'Toothcrimson',
    'color_rgb': (196, 92, 92),
    'color_html': '#c45c5c'},
    '125': {'num': '125',
    'name': '鞓红',
    'name_en': 'Crimsonbelt',
    'color_rgb': (180, 68, 84),
    'color_html': '#b44454'},
    '126': {'num': '126',
    'name': '葡萄褐',
    'name_en': 'Grapeumber',
    'color_rgb': (156, 108, 108),
    'color_html': '#9c6c6c'},
    '127': {'num': '127',
    'name': '蚩尤旗',
    'name_en': 'Chiyounis',
    'color_rgb': (172, 92, 92),
    'color_html': '#ac5c5c'},
    '128': {'num': '128',
    'name': '紫矿',
    'name_en': 'Purpurity',
    'color_rgb': (156, 76, 84),
    'color_html': '#9c4c54'},
    '129': {'num': '129',
    'name': '紫诰',
    'name_en': 'PurpleEdict',
    'color_rgb': (116, 84, 92),
    'color_html': '#74545c'},
    '130': {'num': '130',
    'name': '苏方',
    'name_en': 'Sufang',
    'color_rgb': (132, 68, 76),
    'color_html': '#84444c'},
    '131': {'num': '131',
    'name': '霁红',
    'name_en': 'Crimsonclarity',
    'color_rgb': (124, 68, 76),
    'color_html': '#7c444c'},
    '132': {'num': '132',
    'name': '蜜褐',
    'name_en': 'Honeybrown',
    'color_rgb': (108, 52, 52),
    'color_html': '#6c3434'},
    '133': {'num': '133',
    'name': '福色',
    'name_en': 'Blesshue',
    'color_rgb': (100, 44, 44),
    'color_html': '#642c2c'},
    '134': {'num': '134',
    'name': '黪紫',
    'name_en': 'Duskwine',
    'color_rgb': (204, 116, 164),
    'color_html': '#cc74a4'},
    '135': {'num': '135',
    'name': '龙膏烛',
    'name_en': 'Dragonwaxflame',
    'color_rgb': (220, 132, 164),
    'color_html': '#dc84a4'},
    '136': {'num': '136',
    'name': '苏梅',
    'name_en': 'SumptuousMoon',
    'color_rgb': (220, 116, 148),
    'color_html': '#dc7494'},
    '137': {'num': '137',
    'name': '琅环紫',
    'name_en': 'Lanhuazhi',
    'color_rgb': (204, 92, 132),
    'color_html': '#cc5c84'},
    '138': {'num': '138',
    'name': '胭脂水',
    'name_en': 'RougeWater',
    'color_rgb': (188, 92, 140),
    'color_html': '#bc5c8c'},
    '139': {'num': '139',
    'name': '紫茎屏风',
    'name_en': 'Amethyststem',
    'color_rgb': (163, 99, 131),
    'color_html': '#a36383'},
    '140': {'num': '140',
    'name': '红踯躅',
    'name_en': 'ScarletBloom',
    'color_rgb': (188, 52, 116),
    'color_html': '#bc3474'},
    '141': {'num': '141',
    'name': '胭脂紫',
    'name_en': 'Crimsonhue',
    'color_rgb': (180, 68, 108),
    'color_html': '#b4446c'},
    '142': {'num': '142',
    'name': '魏红',
    'name_en': 'Weihong',
    'color_rgb': (164, 52, 100),
    'color_html': '#a43464'},
    '143': {'num': '143',
    'name': '紫府',
    'name_en': 'Amethystine',
    'color_rgb': (156, 92, 124),
    'color_html': '#9c5c7c'},
    '144': {'num': '144',
    'name': '魏紫',
    'name_en': 'Weizi',
    'color_rgb': (148, 52, 84),
    'color_html': '#943454'},
    '145': {'num': '145',
    'name': '地血',
    'name_en': 'Earthblood',
    'color_rgb': (132, 68, 100),
    'color_html': '#844464'},
    '146': {'num': '146',
    'name': '芥拾紫',
    'name_en': 'Violetglean',
    'color_rgb': (100, 36, 68),
    'color_html': '#642444'},
    '147': {'num': '147',
    'name': '油紫',
    'name_en': 'Oleanthrene',
    'color_rgb': (68, 12, 44),
    'color_html': '#440c2c'},
    '148': {'num': '148',
    'name': '紫薄汗',
    'name_en': 'Amethystine',
    'color_rgb': (188, 164, 204),
    'color_html': '#bca4cc'},
    '149': {'num': '149',
    'name': '退红',
    'name_en': 'Blushrecede',
    'color_rgb': (244, 204, 228),
    'color_html': '#f4cce4'},
    '150': {'num': '150',
    'name': '昌容',
    'name_en': 'Changrong',
    'color_rgb': (220, 196, 228),
    'color_html': '#dcc4e4'},
    '151': {'num': '151',
    'name': '樱花',
    'name_en': 'Cherryblossom',
    'color_rgb': (228, 188, 212),
    'color_html': '#e4bcd4'},
    '152': {'num': '152',
    'name': '丁香',
    'name_en': 'Lilac',
    'color_rgb': (204, 148, 188),
    'color_html': '#cc94bc'},
    '153': {'num': '153',
    'name': '木槿',
    'name_en': 'Muhlenia',
    'color_rgb': (188, 124, 180),
    'color_html': '#bc7cb4'},
    '154': {'num': '154',
    'name': '茈藐',
    'name_en': 'Cimiao',
    'color_rgb': (164, 124, 180),
    'color_html': '#a47cb4'},
    '155': {'num': '155',
    'name': '紫蒲',
    'name_en': 'Purplebulrush',
    'color_rgb': (164, 84, 156),
    'color_html': '#a4549c'},
    '156': {'num': '156',
    'name': '紫紶',
    'name_en': 'Amethystine',
    'color_rgb': (124, 84, 132),
    'color_html': '#7c5484'},
    '157': {'num': '157',
    'name': '拂紫绵',
    'name_en': 'Lavenderfluff',
    'color_rgb': (124, 84, 124),
    'color_html': '#7c547c'},
    '158': {'num': '158',
    'name': '赪紫',
    'name_en': 'Crimsonpurple',
    'color_rgb': (140, 28, 116),
    'color_html': '#8c1c74'},
    '159': {'num': '159',
    'name': '三公子',
    'name_en': 'Thirdson',
    'color_rgb': (100, 60, 116),
    'color_html': '#643c74'},
    '160': {'num': '160',
    'name': '齐紫',
    'name_en': 'Amethystine',
    'color_rgb': (108, 36, 108),
    'color_html': '#6c246c'},
    '161': {'num': '161',
    'name': '凝夜紫',
    'name_en': 'Noctilucence',
    'color_rgb': (68, 36, 84),
    'color_html': '#442454'},
    '162': {'num': '162',
    'name': '石英',
    'name_en': 'Silicarock',
    'color_rgb': (204, 180, 188),
    'color_html': '#ccb4bc'},
    '163': {'num': '163',
    'name': '香炉紫烟',
    'name_en': 'Censoryphume',
    'color_rgb': (211, 203, 211),
    'color_html': '#d3cbd3'},
    '164': {'num': '164',
    'name': '苍烟落照',
    'name_en': 'Crimsonhaze',
    'color_rgb': (203, 179, 179),
    'color_html': '#cbb3b3'},
    '165': {'num': '165',
    'name': '甘石',
    'name_en': 'Ganstone',
    'color_rgb': (188, 180, 180),
    'color_html': '#bcb4b4'},
    '166': {'num': '166',
    'name': '紫菂',
    'name_en': 'Amethystine',
    'color_rgb': (156, 140, 172),
    'color_html': '#9c8cac'},
    '167': {'num': '167',
    'name': '银褐',
    'name_en': 'Silverbrown',
    'color_rgb': (156, 140, 156),
    'color_html': '#9c8c9c'},
    '168': {'num': '168',
    'name': '藕丝褐',
    'name_en': 'Lotusrootbrown',
    'color_rgb': (172, 132, 132),
    'color_html': '#ac8484'},
    '169': {'num': '169',
    'name': '烟红',
    'name_en': 'Crimsonmist',
    'color_rgb': (156, 132, 140),
    'color_html': '#9c848c'},
    '170': {'num': '170',
    'name': '迷楼灰',
    'name_en': 'Mistyloomgray',
    'color_rgb': (148, 132, 140),
    'color_html': '#94848c'},
    '171': {'num': '171',
    'name': '红藤杖',
    'name_en': 'Crimsonvine_staff',
    'color_rgb': (148, 132, 132),
    'color_html': '#948484'},
    '172': {'num': '172',
    'name': '鸦维',
    'name_en': 'Ebonivory',
    'color_rgb': (108, 92, 108),
    'color_html': '#6c5c6c'},
    '173': {'num': '173',
    'name': '玄天',
    'name_en': 'Celestialdeep',
    'color_rgb': (108, 84, 92),
    'color_html': '#6c545c'},
    '174': {'num': '174',
    'name': '烟墨',
    'name_en': 'Inkmist',
    'color_rgb': (92, 76, 84),
    'color_html': '#5c4c54'},
    '175': {'num': '175',
    'name': '紫鼠',
    'name_en': 'VioletSquirrel',
    'color_rgb': (92, 76, 84),
    'color_html': '#5c4c54'},
    '176': {'num': '176',
    'name': '栀子',
    'name_en': 'Gardenia',
    'color_rgb': (252, 196, 60),
    'color_html': '#fcc43c'},
    '177': {'num': '177',
    'name': '黄白游',
    'name_en': 'Aureolus',
    'color_rgb': (252, 244, 156),
    'color_html': '#fcf49c'},
    '178': {'num': '178',
    'name': '松花',
    'name_en': 'Pinebloom',
    'color_rgb': (252, 236, 108),
    'color_html': '#fcec6c'},
    '179': {'num': '179',
    'name': '缃叶',
    'name_en': 'Verdantleaf',
    'color_rgb': (236, 212, 84),
    'color_html': '#ecd454'},
    '180': {'num': '180',
    'name': '黄栗留',
    'name_en': 'Orchardflame',
    'color_rgb': (252, 220, 92),
    'color_html': '#fcdc5c'},
    '181': {'num': '181',
    'name': '嫩鹅黄',
    'name_en': 'Gosssling_yellow',
    'color_rgb': (244, 204, 100),
    'color_html': '#f4cc64'},
    '182': {'num': '182',
    'name': '黄河琉璃',
    'name_en': 'Aurelianriver',
    'color_rgb': (227, 171, 75),
    'color_html': '#e3ab4b'},
    '183': {'num': '183',
    'name': '杏子',
    'name_en': 'Apricot',
    'color_rgb': (220, 148, 52),
    'color_html': '#dc9434'},
    '184': {'num': '184',
    'name': '红友',
    'name_en': 'Crimsonfriend',
    'color_rgb': (220, 140, 60),
    'color_html': '#dc8c3c'},
    '185': {'num': '185',
    'name': '库金',
    'name_en': 'Crescentia',
    'color_rgb': (228, 140, 60),
    'color_html': '#e48c3c'},
    '186': {'num': '186',
    'name': '鞠衣',
    'name_en': 'Chrysanthemumcloak',
    'color_rgb': (212, 164, 52),
    'color_html': '#d4a434'},
    '187': {'num': '187',
    'name': '黄不老',
    'name_en': 'Eternalgreen',
    'color_rgb': (220, 156, 52),
    'color_html': '#dc9c34'},
    '188': {'num': '188',
    'name': '郁金裙',
    'name_en': 'Saffronskirt',
    'color_rgb': (212, 132, 52),
    'color_html': '#d48434'},
    '189': {'num': '189',
    'name': '露褐',
    'name_en': 'Dewhued',
    'color_rgb': (188, 132, 84),
    'color_html': '#bc8454'},
    '190': {'num': '190',
    'name': '蛾黄',
    'name_en': 'Eryellow',
    'color_rgb': (188, 140, 44),
    'color_html': '#bc8c2c'},
    '191': {'num': '191',
    'name': '光明砂',
    'name_en': 'LuminousSilt',
    'color_rgb': (204, 92, 36),
    'color_html': '#cc5c24'},
    '192': {'num': '192',
    'name': '柘黄',
    'name_en': 'Topazgold',
    'color_rgb': (196, 124, 20),
    'color_html': '#c47c14'},
    '193': {'num': '193',
    'name': '媚蝶',
    'name_en': 'Enchantbutterfly',
    'color_rgb': (188, 108, 52),
    'color_html': '#bc6c34'},
    '194': {'num': '194',
    'name': '黄流',
    'name_en': 'Aurestream',
    'color_rgb': (156, 100, 36),
    'color_html': '#9c6424'},
    '195': {'num': '195',
    'name': '韎韐',
    'name_en': 'Eleganceweave',
    'color_rgb': (156, 84, 36),
    'color_html': '#9c5424'},
    '196': {'num': '196',
    'name': '九斤黄',
    'name_en': 'Ninegold',
    'color_rgb': (220, 180, 124),
    'color_html': '#dcb47c'},
    '197': {'num': '197',
    'name': '弗肯红',
    'name_en': 'Florvermilion',
    'color_rgb': (236, 220, 196),
    'color_html': '#ecdcc4'},
    '198': {'num': '198',
    'name': '赤璋',
    'name_en': 'Crimsontablet',
    'color_rgb': (228, 196, 156),
    'color_html': '#e4c49c'},
    '199': {'num': '199',
    'name': '如梦令',
    'name_en': 'Dreamverse',
    'color_rgb': (220, 188, 156),
    'color_html': '#dcbc9c'},
    '200': {'num': '200',
    'name': '茧色',
    'name_en': 'Cocoonshade',
    'color_rgb': (196, 164, 108),
    'color_html': '#c4a46c'},
    '201': {'num': '201',
    'name': '芸黄',
    'name_en': 'Gleamgold',
    'color_rgb': (212, 164, 108),
    'color_html': '#d4a46c'},
    '202': {'num': '202',
    'name': '椒房',
    'name_en': 'Pepperclove',
    'color_rgb': (220, 156, 92),
    'color_html': '#dc9c5c'},
    '203': {'num': '203',
    'name': '金埒',
    'name_en': 'GildedBridle',
    'color_rgb': (188, 148, 84),
    'color_html': '#bc9454'},
    '204': {'num': '204',
    'name': '雌黄',
    'name_en': 'Orpiment',
    'color_rgb': (180, 140, 76),
    'color_html': '#b48c4c'},
    '205': {'num': '205',
    'name': '密陀僧',
    'name_en': 'Mitalite',
    'color_rgb': (180, 148, 76),
    'color_html': '#b4944c'},
    '206': {'num': '206',
    'name': '大块',
    'name_en': 'Massivepiece',
    'color_rgb': (188, 164, 132),
    'color_html': '#bca484'},
    '207': {'num': '207',
    'name': '蜜合',
    'name_en': 'Honeyblend',
    'color_rgb': (220, 212, 196),
    'color_html': '#dcd4c4'},
    '208': {'num': '208',
    'name': '地籁',
    'name_en': 'Earthwhisper',
    'color_rgb': (220, 204, 180),
    'color_html': '#dcccb4'},
    '209': {'num': '209',
    'name': '仙米',
    'name_en': 'CelestialRice',
    'color_rgb': (204, 204, 172),
    'color_html': '#ccccac'},
    '210': {'num': '210',
    'name': '假山南',
    'name_en': 'Southfalsescarp',
    'color_rgb': (212, 196, 164),
    'color_html': '#d4c4a4'},
    '211': {'num': '211',
    'name': '黄粱',
    'name_en': 'GoldenGrainDream',
    'color_rgb': (196, 180, 156),
    'color_html': '#c4b49c'},
    '212': {'num': '212',
    'name': '石蜜',
    'name_en': 'Rockhoney',
    'color_rgb': (212, 188, 140),
    'color_html': '#d4bc8c'},
    '213': {'num': '213',
    'name': '紫花布',
    'name_en': 'Amethystweave',
    'color_rgb': (188, 164, 140),
    'color_html': '#bca48c'},
    '214': {'num': '214',
    'name': '黄封',
    'name_en': 'Amaranthine',
    'color_rgb': (204, 180, 116),
    'color_html': '#ccb474'},
    '215': {'num': '215',
    'name': '养生主',
    'name_en': 'Nourishlife',
    'color_rgb': (180, 156, 124),
    'color_html': '#b49c7c'},
    '216': {'num': '216',
    'name': '沙场',
    'name_en': 'Battlescene',
    'color_rgb': (188, 164, 116),
    'color_html': '#bca474'},
    '217': {'num': '217',
    'name': '黄螺',
    'name_en': 'Goldenwhelk',
    'color_rgb': (180, 164, 124),
    'color_html': '#b4a47c'},
    '218': {'num': '218',
    'name': '蒸栗',
    'name_en': 'Steamedchestnut',
    'color_rgb': (164, 140, 92),
    'color_html': '#a48c5c'},
    '219': {'num': '219',
    'name': '巨吕',
    'name_en': 'Titanlyre',
    'color_rgb': (172, 140, 92),
    'color_html': '#ac8c5c'},
    '220': {'num': '220',
    'name': '降真香',
    'name_en': 'Jiangzhenxiang',
    'color_rgb': (156, 132, 92),
    'color_html': '#9c845c'},
    '221': {'num': '221',
    'name': '大云',
    'name_en': 'Megalonimbus',
    'color_rgb': (148, 124, 76),
    'color_html': '#947c4c'},
    '222': {'num': '222',
    'name': '吉金',
    'name_en': 'Auremetal',
    'color_rgb': (140, 108, 68),
    'color_html': '#8c6c44'},
    '223': {'num': '223',
    'name': '远志',
    'name_en': 'Aspiration',
    'color_rgb': (132, 100, 60),
    'color_html': '#84643c'},
    '224': {'num': '224',
    'name': '射干',
    'name_en': 'Belamcanda',
    'color_rgb': (124, 100, 60),
    'color_html': '#7c643c'},
    '225': {'num': '225',
    'name': '油葫芦',
    'name_en': 'Oilcricket',
    'color_rgb': (100, 76, 52),
    'color_html': '#644c34'},
    '226': {'num': '226',
    'name': '龙战',
    'name_en': 'Dragonclash',
    'color_rgb': (92, 68, 36),
    'color_html': '#5c4424'},
    '227': {'num': '227',
    'name': '緅絺',
    'name_en': 'Zozichiffon',
    'color_rgb': (132, 76, 44),
    'color_html': '#844c2c'},
    '228': {'num': '228',
    'name': '葭灰',
    'name_en': 'ReedAsh',
    'color_rgb': (188, 180, 172),
    'color_html': '#bcb4ac'},
    '229': {'num': '229',
    'name': '珠子褐',
    'name_en': 'Pearlbrown',
    'color_rgb': (188, 172, 156),
    'color_html': '#bcac9c'},
    '230': {'num': '230',
    'name': '黄埃',
    'name_en': 'Aureolearth',
    'color_rgb': (180, 148, 116),
    'color_html': '#b49474'},
    '231': {'num': '231',
    'name': '沉香',
    'name_en': 'Agarwood',
    'color_rgb': (156, 132, 108),
    'color_html': '#9c846c'},
    '232': {'num': '232',
    'name': '明茶褐',
    'name_en': 'Amberlight',
    'color_rgb': (156, 132, 108),
    'color_html': '#9c846c'},
    '233': {'num': '233',
    'name': '荆褐',
    'name_en': 'Thornumber',
    'color_rgb': (148, 108, 76),
    'color_html': '#946c4c'},
    '234': {'num': '234',
    'name': '驼褐',
    'name_en': 'Camelochre',
    'color_rgb': (124, 92, 60),
    'color_html': '#7c5c3c'},
    '235': {'num': '235',
    'name': '缊韨',
    'name_en': 'Mystiflux',
    'color_rgb': (156, 76, 52),
    'color_html': '#9c4c34'},
    '236': {'num': '236',
    'name': '棠梨褐',
    'name_en': 'Tangleebrown',
    'color_rgb': (148, 92, 68),
    'color_html': '#945c44'},
    '237': {'num': '237',
    'name': '檀褐',
    'name_en': 'Tanhued',
    'color_rgb': (148, 84, 52),
    'color_html': '#945434'},
    '238': {'num': '238',
    'name': '朱石栗',
    'name_en': 'Crimsonstonechestnut',
    'color_rgb': (132, 76, 44),
    'color_html': '#844c2c'},
    '239': {'num': '239',
    'name': '紫瓯',
    'name_en': 'AmethystCup',
    'color_rgb': (124, 68, 28),
    'color_html': '#7c441c'},
    '240': {'num': '240',
    'name': '栗壳',
    'name_en': 'Chestnutshell',
    'color_rgb': (116, 84, 60),
    'color_html': '#74543c'},
    '241': {'num': '241',
    'name': '麝香褐',
    'name_en': 'Muskbrown',
    'color_rgb': (108, 76, 60),
    'color_html': '#6c4c3c'},
    '242': {'num': '242',
    'name': '椒褐',
    'name_en': 'Peppervern',
    'color_rgb': (116, 68, 60),
    'color_html': '#74443c'},
    '243': {'num': '243',
    'name': '枣褐',
    'name_en': 'Jujubrown',
    'color_rgb': (108, 52, 28),
    'color_html': '#6c341c'},
    '244': {'num': '244',
    'name': '目童子',
    'name_en': 'Eyeling',
    'color_rgb': (92, 52, 36),
    'color_html': '#5c3424'},
    '245': {'num': '245',
    'name': '青骊',
    'name_en': 'Azuresteed',
    'color_rgb': (68, 36, 20),
    'color_html': '#442414'},
    '246': {'num': '246',
    'name': '丁香褐',
    'name_en': 'Lilacbrown',
    'color_rgb': (188, 148, 132),
    'color_html': '#bc9484'},
    '247': {'num': '247',
    'name': '肉红',
    'name_en': 'Carnescent',
    'color_rgb': (220, 196, 188),
    'color_html': '#dcc4bc'},
    '248': {'num': '248',
    'name': '夏籥',
    'name_en': 'Summerflute',
    'color_rgb': (212, 172, 156),
    'color_html': '#d4ac9c'},
    '249': {'num': '249',
    'name': '檀唇',
    'name_en': 'Rosekiss',
    'color_rgb': (220, 156, 140),
    'color_html': '#dc9c8c'},
    '250': {'num': '250',
    'name': '紫磨金',
    'name_en': 'Amethystgold',
    'color_rgb': (188, 132, 108),
    'color_html': '#bc846c'},
    '251': {'num': '251',
    'name': '檀色',
    'name_en': 'Rosewood',
    'color_rgb': (180, 108, 92),
    'color_html': '#b46c5c'},
    '252': {'num': '252',
    'name': '鹰背褐',
    'name_en': 'Eaglebackbrown',
    'color_rgb': (140, 108, 92),
    'color_html': '#8c6c5c'},
    '253': {'num': '253',
    'name': '赭罗',
    'name_en': 'Zhoran',
    'color_rgb': (156, 100, 84),
    'color_html': '#9c6454'},
    '254': {'num': '254',
    'name': '老僧衣',
    'name_en': 'Elderrobe',
    'color_rgb': (164, 92, 68),
    'color_html': '#a45c44'},
    '255': {'num': '255',
    'name': '姜黄',
    'name_en': 'Saffronspice',
    'color_rgb': (212, 196, 100),
    'color_html': '#d4c464'},
    '256': {'num': '256',
    'name': '半见',
    'name_en': 'Semiview',
    'color_rgb': (252, 252, 196),
    'color_html': '#fcfcc4'},
    '257': {'num': '257',
    'name': '断肠',
    'name_en': 'Heartbreak',
    'color_rgb': (236, 236, 196),
    'color_html': '#ececc4'},
    '258': {'num': '258',
    'name': '葱青',
    'name_en': 'Verdigris',
    'color_rgb': (236, 244, 188),
    'color_html': '#ecf4bc'},
    '259': {'num': '259',
    'name': '女贞黄',
    'name_en': 'LigustrumFlava',
    'color_rgb': (244, 236, 172),
    'color_html': '#f4ecac'},
    '260': {'num': '260',
    'name': '莺儿',
    'name_en': 'Nightingale',
    'color_rgb': (236, 228, 172),
    'color_html': '#ece4ac'},
    '261': {'num': '261',
    'name': '桑蕾',
    'name_en': 'Mulberryray',
    'color_rgb': (236, 220, 156),
    'color_html': '#ecdc9c'},
    '262': {'num': '262',
    'name': '绢纨',
    'name_en': 'Silkgossamer',
    'color_rgb': (236, 228, 148),
    'color_html': '#ece494'},
    '263': {'num': '263',
    'name': '少艾',
    'name_en': 'Youthbloom',
    'color_rgb': (228, 236, 156),
    'color_html': '#e4ec9c'},
    '264': {'num': '264',
    'name': '绮钱',
    'name_en': 'Gossamercoin',
    'color_rgb': (220, 220, 140),
    'color_html': '#dcdc8c'},
    '265': {'num': '265',
    'name': '翠樽',
    'name_en': 'Jadeflagon',
    'color_rgb': (204, 212, 116),
    'color_html': '#ccd474'},
    '266': {'num': '266',
    'name': '田赤',
    'name_en': 'Terracrimson',
    'color_rgb': (228, 212, 132),
    'color_html': '#e4d484'},
    '267': {'num': '267',
    'name': '禹余粮',
    'name_en': 'Yuyuliang',
    'color_rgb': (228, 212, 124),
    'color_html': '#e4d47c'},
    '268': {'num': '268',
    'name': '姚黄',
    'name_en': 'YaoHuang',
    'color_rgb': (212, 188, 68),
    'color_html': '#d4bc44'},
    '269': {'num': '269',
    'name': '太一余粮',
    'name_en': 'PrimordialSustenance',
    'color_rgb': (211, 179, 91),
    'color_html': '#d3b35b'},
    '270': {'num': '270',
    'name': '栾华',
    'name_en': 'Luanhua',
    'color_rgb': (196, 172, 92),
    'color_html': '#c4ac5c'},
    '271': {'num': '271',
    'name': '秋香',
    'name_en': 'AutumnBloom',
    'color_rgb': (188, 156, 68),
    'color_html': '#bc9c44'},
    '272': {'num': '272',
    'name': '大赤',
    'name_en': 'Crimsonheart',
    'color_rgb': (172, 148, 76),
    'color_html': '#ac944c'},
    '273': {'num': '273',
    'name': '苍黄',
    'name_en': 'Canghuang',
    'color_rgb': (180, 164, 20),
    'color_html': '#b4a414'},
    '274': {'num': '274',
    'name': '老茯神',
    'name_en': 'Elderwood',
    'color_rgb': (172, 132, 52),
    'color_html': '#ac8434'},
    '275': {'num': '275',
    'name': '流黄',
    'name_en': 'Aureolin',
    'color_rgb': (140, 116, 68),
    'color_html': '#8c7444'},
    '276': {'num': '276',
    'name': '青白玉',
    'name_en': 'Cyanjade',
    'color_rgb': (204, 196, 164),
    'color_html': '#ccc4a4'},
    '277': {'num': '277',
    'name': '玉色',
    'name_en': 'Jadehue',
    'color_rgb': (236, 228, 212),
    'color_html': '#ece4d4'},
    '278': {'num': '278',
    'name': '骨缥',
    'name_en': 'Bonegossamer',
    'color_rgb': (236, 228, 196),
    'color_html': '#ece4c4'},
    '279': {'num': '279',
    'name': '黄润',
    'name_en': 'Amberglow',
    'color_rgb': (220, 212, 188),
    'color_html': '#dcd4bc'},
    '280': {'num': '280',
    'name': '缣缃',
    'name_en': 'Silkgilt',
    'color_rgb': (212, 204, 164),
    'color_html': '#d4cca4'},
    '281': {'num': '281',
    'name': '佩玖',
    'name_en': 'Peachjewel',
    'color_rgb': (172, 156, 140),
    'color_html': '#ac9c8c'},
    '282': {'num': '282',
    'name': '玄校',
    'name_en': 'Mysticlore',
    'color_rgb': (172, 164, 132),
    'color_html': '#aca484'},
    '283': {'num': '283',
    'name': '黄琮',
    'name_en': 'Ambercite',
    'color_rgb': (156, 140, 108),
    'color_html': '#9c8c6c'},
    '284': {'num': '284',
    'name': '石莲褐',
    'name_en': 'Stonebloom',
    'color_rgb': (148, 140, 124),
    'color_html': '#948c7c'},
    '285': {'num': '285',
    'name': '绿豆褐',
    'name_en': 'Mungumber',
    'color_rgb': (148, 140, 108),
    'color_html': '#948c6c'},
    '286': {'num': '286',
    'name': '綟绶',
    'name_en': 'Twisilken',
    'color_rgb': (116, 108, 76),
    'color_html': '#746c4c'},
    '287': {'num': '287',
    'name': '茶色',
    'name_en': 'Teacolored',
    'color_rgb': (140, 116, 84),
    'color_html': '#8c7454'},
    '288': {'num': '288',
    'name': '濯绛',
    'name_en': 'Scarletwash',
    'color_rgb': (124, 108, 100),
    'color_html': '#7c6c64'},
    '289': {'num': '289',
    'name': '黑朱',
    'name_en': 'Ebonyvermilion',
    'color_rgb': (116, 108, 92),
    'color_html': '#746c5c'},
    '290': {'num': '290',
    'name': '冥色',
    'name_en': 'Twilightshade',
    'color_rgb': (100, 92, 76),
    'color_html': '#645c4c'},
    '291': {'num': '291',
    'name': '伽罗',
    'name_en': 'Jialuo',
    'color_rgb': (108, 92, 60),
    'color_html': '#6c5c3c'},
    '292': {'num': '292',
    'name': '苍艾',
    'name_en': 'Cyanheart',
    'color_rgb': (92, 76, 60),
    'color_html': '#5c4c3c'},
    '293': {'num': '293',
    'name': '群青',
    'name_en': 'Ultramarine',
    'color_rgb': (44, 92, 164),
    'color_html': '#2c5ca4'},
    '294': {'num': '294',
    'name': '碧落',
    'name_en': 'Azurehaven',
    'color_rgb': (172, 212, 236),
    'color_html': '#acd4ec'},
    '295': {'num': '295',
    'name': '晴山',
    'name_en': 'Sunnypeak',
    'color_rgb': (164, 188, 220),
    'color_html': '#a4bcdc'},
    '296': {'num': '296',
    'name': '品月',
    'name_en': 'MoonSavor',
    'color_rgb': (140, 172, 204),
    'color_html': '#8caccc'},
    '297': {'num': '297',
    'name': '窃蓝',
    'name_en': 'Stealblue',
    'color_rgb': (140, 172, 220),
    'color_html': '#8cacdc'},
    '298': {'num': '298',
    'name': '挼蓝',
    'name_en': 'Azuremeld',
    'color_rgb': (108, 156, 196),
    'color_html': '#6c9cc4'},
    '299': {'num': '299',
    'name': '监德',
    'name_en': 'Virtuguard',
    'color_rgb': (108, 148, 204),
    'color_html': '#6c94cc'},
    '300': {'num': '300',
    'name': '苍苍',
    'name_en': 'Ethereal',
    'color_rgb': (92, 116, 188),
    'color_html': '#5c74bc'},
    '301': {'num': '301',
    'name': '孔雀蓝',
    'name_en': 'Peacoblue',
    'color_rgb': (76, 148, 196),
    'color_html': '#4c94c4'},
    '302': {'num': '302',
    'name': '青冥',
    'name_en': 'Azuredeep',
    'color_rgb': (52, 116, 172),
    'color_html': '#3474ac'},
    '303': {'num': '303',
    'name': '柔蓝',
    'name_en': 'Cerulean',
    'color_rgb': (20, 108, 156),
    'color_html': '#146c9c'},
    '304': {'num': '304',
    'name': '碧城',
    'name_en': 'Jadecity',
    'color_rgb': (20, 84, 124),
    'color_html': '#14547c'},
    '305': {'num': '305',
    'name': '蓝采和',
    'name_en': 'Lanciferous',
    'color_rgb': (4, 68, 108),
    'color_html': '#04446c'},
    '306': {'num': '306',
    'name': '细宇',
    'name_en': 'Microcosm',
    'color_rgb': (4, 60, 116),
    'color_html': '#043c74'},
    '307': {'num': '307',
    'name': '帝释青',
    'name_en': 'Indigocyan',
    'color_rgb': (4, 52, 100),
    'color_html': '#043464'},
    '308': {'num': '308',
    'name': '佛头青',
    'name_en': 'BuddhaAzure',
    'color_rgb': (28, 52, 92),
    'color_html': '#1c345c'},
    '309': {'num': '309',
    'name': '骐麟',
    'name_en': 'Qilin',
    'color_rgb': (20, 36, 76),
    'color_html': '#14244c'},
    '310': {'num': '310',
    'name': '花青',
    'name_en': 'Florazure',
    'color_rgb': (28, 44, 68),
    'color_html': '#1c2c44'},
    '311': {'num': '311',
    'name': '优昙瑞',
    'name_en': 'Udannia',
    'color_rgb': (100, 92, 172),
    'color_html': '#645cac'},
    '312': {'num': '312',
    'name': '暮山紫',
    'name_en': 'VespertinePurple',
    'color_rgb': (164, 172, 212),
    'color_html': '#a4acd4'},
    '313': {'num': '313',
    'name': '紫苑',
    'name_en': 'Asterflower',
    'color_rgb': (116, 124, 188),
    'color_html': '#747cbc'},
    '314': {'num': '314',
    'name': '延维',
    'name_en': 'Eternweave',
    'color_rgb': (76, 76, 156),
    'color_html': '#4c4c9c'},
    '315': {'num': '315',
    'name': '曾青',
    'name_en': 'Verdigris',
    'color_rgb': (84, 84, 100),
    'color_html': '#545464'},
    '316': {'num': '316',
    'name': '青黛',
    'name_en': 'Azurehue',
    'color_rgb': (68, 68, 92),
    'color_html': '#44445c'},
    '317': {'num': '317',
    'name': '䒌靘',
    'name_en': 'Ethereal',
    'color_rgb': (68, 68, 92),
    'color_html': '#44445c'},
    '318': {'num': '318',
    'name': '缪琳',
    'name_en': 'Miolin',
    'color_rgb': (52, 52, 68),
    'color_html': '#343444'},
    '319': {'num': '319',
    'name': '绀蝶',
    'name_en': 'Azurebutterfly',
    'color_rgb': (44, 44, 60),
    'color_html': '#2c2c3c'},
    '320': {'num': '320',
    'name': '獭见',
    'name_en': 'Otterview',
    'color_rgb': (20, 28, 44),
    'color_html': '#141c2c'},
    '321': {'num': '321',
    'name': '天水碧',
    'name_en': 'Aquaheavenly',
    'color_rgb': (92, 164, 172),
    'color_html': '#5ca4ac'},
    '322': {'num': '322',
    'name': '井天',
    'name_en': 'Celestialwell',
    'color_rgb': (164, 204, 204),
    'color_html': '#a4cccc'},
    '323': {'num': '323',
    'name': '云门',
    'name_en': 'Cloudgate',
    'color_rgb': (164, 212, 228),
    'color_html': '#a4d4e4'},
    '324': {'num': '324',
    'name': '西子',
    'name_en': 'Xizi',
    'color_rgb': (132, 196, 204),
    'color_html': '#84c4cc'},
    '325': {'num': '325',
    'name': '正青',
    'name_en': 'Verdancy',
    'color_rgb': (108, 172, 172),
    'color_html': '#6cacac'},
    '326': {'num': '326',
    'name': '扁青',
    'name_en': 'Azureblade',
    'color_rgb': (84, 148, 148),
    'color_html': '#549494'},
    '327': {'num': '327',
    'name': '法翠',
    'name_en': 'Jadefire',
    'color_rgb': (20, 140, 148),
    'color_html': '#148c94'},
    '328': {'num': '328',
    'name': '吐绶蓝',
    'name_en': 'Azureplume',
    'color_rgb': (68, 132, 164),
    'color_html': '#4484a4'},
    '329': {'num': '329',
    'name': '鱼师青',
    'name_en': 'Fishmasterblue',
    'color_rgb': (52, 124, 140),
    'color_html': '#347c8c'},
    '330': {'num': '330',
    'name': '软翠',
    'name_en': 'Melliflueve',
    'color_rgb': (4, 108, 132),
    'color_html': '#046c84'},
    '331': {'num': '331',
    'name': '青緺',
    'name_en': 'Azureweave',
    'color_rgb': (41, 72, 82),
    'color_html': '#294852'},
    '332': {'num': '332',
    'name': '螺子黛',
    'name_en': 'SpiralIndigo',
    'color_rgb': (20, 60, 60),
    'color_html': '#143c3c'},
    '333': {'num': '333',
    'name': '二绿',
    'name_en': 'Emeraude',
    'color_rgb': (92, 164, 156),
    'color_html': '#5ca49c'},
    '334': {'num': '334',
    'name': '繱犗',
    'name_en': 'Silkenbound',
    'color_rgb': (140, 188, 188),
    'color_html': '#8cbcbc'},
    '335': {'num': '335',
    'name': '铜青',
    'name_en': 'CupricPatina',
    'color_rgb': (60, 140, 132),
    'color_html': '#3c8c84'},
    '336': {'num': '336',
    'name': '青雘',
    'name_en': 'Azureglow',
    'color_rgb': (4, 116, 116),
    'color_html': '#047474'},
    '337': {'num': '337',
    'name': '䌦色',
    'name_en': 'Silkglow',
    'color_rgb': (36, 108, 108),
    'color_html': '#246c6c'},
    '338': {'num': '338',
    'name': '石绿',
    'name_en': 'Jadegreen',
    'color_rgb': (36, 108, 100),
    'color_html': '#246c64'},
    '339': {'num': '339',
    'name': '竹月',
    'name_en': 'Bamboonight',
    'color_rgb': (124, 156, 172),
    'color_html': '#7c9cac'},
    '340': {'num': '340',
    'name': '月白',
    'name_en': 'Moonlit',
    'color_rgb': (212, 228, 236),
    'color_html': '#d4e4ec'},
    '341': {'num': '341',
    'name': '素采',
    'name_en': 'Luminance',
    'color_rgb': (212, 220, 228),
    'color_html': '#d4dce4'},
    '342': {'num': '342',
    'name': '星郎',
    'name_en': 'Stellarbright',
    'color_rgb': (188, 212, 228),
    'color_html': '#bcd4e4'},
    '343': {'num': '343',
    'name': '影青',
    'name_en': 'Celadonshade',
    'color_rgb': (188, 204, 212),
    'color_html': '#bcccd4'},
    '344': {'num': '344',
    'name': '逍遥游',
    'name_en': 'Freerove',
    'color_rgb': (180, 188, 196),
    'color_html': '#b4bcc4'},
    '345': {'num': '345',
    'name': '白青',
    'name_en': 'Azuregreen',
    'color_rgb': (156, 180, 196),
    'color_html': '#9cb4c4'},
    '346': {'num': '346',
    'name': '青鸾',
    'name_en': 'Azureon',
    'color_rgb': (156, 164, 180),
    'color_html': '#9ca4b4'},
    '347': {'num': '347',
    'name': '东方既白',
    'name_en': 'Easterwhite',
    'color_rgb': (139, 163, 195),
    'color_html': '#8ba3c3'},
    '348': {'num': '348',
    'name': '秋蓝',
    'name_en': 'Autumnbloom',
    'color_rgb': (124, 148, 156),
    'color_html': '#7c949c'},
    '349': {'num': '349',
    'name': '空青',
    'name_en': 'Voidazure',
    'color_rgb': (100, 140, 156),
    'color_html': '#648c9c'},
    '350': {'num': '350',
    'name': '太师青',
    'name_en': 'EminenceAzure',
    'color_rgb': (84, 116, 140),
    'color_html': '#54748c'},
    '351': {'num': '351',
    'name': '菘蓝',
    'name_en': 'Isatisroot',
    'color_rgb': (108, 124, 140),
    'color_html': '#6c7c8c'},
    '352': {'num': '352',
    'name': '育阳染',
    'name_en': 'Nuriscent',
    'color_rgb': (84, 100, 116),
    'color_html': '#546474'},
    '353': {'num': '353',
    'name': '青雀头黛',
    'name_en': 'Azurecrest',
    'color_rgb': (52, 76, 108),
    'color_html': '#344c6c'},
    '354': {'num': '354',
    'name': '霁蓝',
    'name_en': 'Azureclarity',
    'color_rgb': (60, 68, 84),
    'color_html': '#3c4454'},
    '355': {'num': '355',
    'name': '瑾瑜',
    'name_en': 'Jinyu',
    'color_rgb': (28, 36, 52),
    'color_html': '#1c2434'},
    '356': {'num': '356',
    'name': '缟羽',
    'name_en': 'Plumefrost',
    'color_rgb': (236, 236, 236),
    'color_html': '#ececec'},
    '357': {'num': '357',
    'name': '山矾',
    'name_en': 'Mountflora',
    'color_rgb': (244, 244, 244),
    'color_html': '#f4f4f4'},
    '358': {'num': '358',
    'name': '浅云',
    'name_en': 'Mistlight',
    'color_rgb': (236, 236, 244),
    'color_html': '#ececf4'},
    '359': {'num': '359',
    'name': '凝脂',
    'name_en': 'Lustrouscream',
    'color_rgb': (244, 244, 236),
    'color_html': '#f4f4ec'},
    '360': {'num': '360',
    'name': '皦玉',
    'name_en': 'Lustrousjade',
    'color_rgb': (236, 236, 236),
    'color_html': '#ececec'},
    '361': {'num': '361',
    'name': '玉頩',
    'name_en': 'Jadeblush',
    'color_rgb': (236, 228, 228),
    'color_html': '#ece4e4'},
    '362': {'num': '362',
    'name': '二目鱼',
    'name_en': 'Twogaze',
    'color_rgb': (220, 228, 220),
    'color_html': '#dce4dc'},
    '363': {'num': '363',
    'name': '韶粉',
    'name_en': 'Shaurin',
    'color_rgb': (228, 228, 212),
    'color_html': '#e4e4d4'},
    '364': {'num': '364',
    'name': '香皮',
    'name_en': 'Aromaskin',
    'color_rgb': (220, 212, 196),
    'color_html': '#dcd4c4'},
    '365': {'num': '365',
    'name': '明月珰',
    'name_en': 'Moonjewel',
    'color_rgb': (212, 212, 204),
    'color_html': '#d4d4cc'},
    '366': {'num': '366',
    'name': '藕丝秋半',
    'name_en': 'Autumngossamer',
    'color_rgb': (211, 203, 195),
    'color_html': '#d3cbc3'},
    '367': {'num': '367',
    'name': '云母',
    'name_en': 'Micaflora',
    'color_rgb': (196, 188, 180),
    'color_html': '#c4bcb4'},
    '368': {'num': '368',
    'name': '米汤娇',
    'name_en': 'Riceliqueur',
    'color_rgb': (236, 236, 220),
    'color_html': '#ececdc'},
    '369': {'num': '369',
    'name': '酂白',
    'name_en': 'Zanbai',
    'color_rgb': (244, 252, 228),
    'color_html': '#f4fce4'},
    '370': {'num': '370',
    'name': '吉量',
    'name_en': 'Aurequine',
    'color_rgb': (236, 236, 220),
    'color_html': '#ececdc'},
    '371': {'num': '371',
    'name': '天球',
    'name_en': 'CelestialSphere',
    'color_rgb': (228, 220, 196),
    'color_html': '#e4dcc4'},
    '372': {'num': '372',
    'name': '霜地',
    'name_en': 'Frostland',
    'color_rgb': (196, 196, 180),
    'color_html': '#c4c4b4'},
    '373': {'num': '373',
    'name': '余白',
    'name_en': 'Whitespace',
    'color_rgb': (204, 204, 196),
    'color_html': '#ccccc4'},
    '374': {'num': '374',
    'name': '溶溶月',
    'name_en': 'Lunaria',
    'color_rgb': (188, 196, 188),
    'color_html': '#bcc4bc'},
    '375': {'num': '375',
    'name': '月魄',
    'name_en': 'Lunaraether',
    'color_rgb': (180, 180, 180),
    'color_html': '#b4b4b4'},
    '376': {'num': '376',
    'name': '冻缥',
    'name_en': 'Frostwhisper',
    'color_rgb': (188, 196, 180),
    'color_html': '#bcc4b4'},
    '377': {'num': '377',
    'name': '草白',
    'name_en': 'Whitengrass',
    'color_rgb': (188, 196, 172),
    'color_html': '#bcc4ac'},
    '378': {'num': '378',
    'name': '不皂',
    'name_en': 'Unsullied',
    'color_rgb': (164, 172, 164),
    'color_html': '#a4aca4'},
    '379': {'num': '379',
    'name': '绍衣',
    'name_en': 'Silkgarment',
    'color_rgb': (172, 164, 156),
    'color_html': '#aca49c'},
    '380': {'num': '380',
    'name': '雷雨垂',
    'name_en': 'Thunderdrip',
    'color_rgb': (124, 124, 124),
    'color_html': '#7c7c7c'},
    '381': {'num': '381',
    'name': '石涅',
    'name_en': 'Stoneglow',
    'color_rgb': (108, 108, 100),
    'color_html': '#6c6c64'},
    '382': {'num': '382',
    'name': '墨黪',
    'name_en': 'Inkshadow',
    'color_rgb': (92, 84, 76),
    'color_html': '#5c544c'},
    '383': {'num': '383',
    'name': '驖骊',
    'name_en': 'Ebonyglow',
    'color_rgb': (68, 68, 60),
    'color_html': '#44443c'},
    '384': {'num': '384',
    'name': '京元',
    'name_en': 'Jingyuan',
    'color_rgb': (52, 52, 44),
    'color_html': '#34342c'}}
    return Forbiddencity


#from ._forbiddencity import get_forbidden
Forbiddencity = get_forbidden()

@register_function(
    aliases=["禁城色彩", "ForbiddenCity", "forbidden_city", "中国传统色", "故宫色彩"],
    category="pl",
    description="Traditional Chinese color palette from Forbidden City with 384 colors",
    examples=[
        "# Initialize Forbidden City color palette",
        "fb = ov.pl.ForbiddenCity()",
        "# Get specific color by name",
        "color = fb.get_color('凝夜紫')",
        "# Get RGB values",
        "rgb = fb.get_color_rgb('群青')",
        "# Create custom colormap",
        "colors = [fb.get_color_rgb('群青'), fb.get_color_rgb('半见'), fb.get_color_rgb('丹缽')]",
        "cmap = fb.get_cmap_seg(colors)",
        "# Use in plotting",
        "ov.pl.embedding(adata, basis='X_umap', color='cell_type',",
        "                palette=fb.red[:])"
    ],
    related=["pl.embedding", "pl.volcano", "pl.boxplot"]
)
class ForbiddenCity:

    green1=Forbidden_Cmap(range(1,19))
    green2=Forbidden_Cmap(range(19,41))
    green3=Forbidden_Cmap(range(41,62))
    red1=Forbidden_Cmap(range(62,77))
    red2=Forbidden_Cmap(range(77,104))
    pink1=Forbidden_Cmap(range(104,134))
    pink2=Forbidden_Cmap(range(134,148))
    purple1=Forbidden_Cmap(range(148,162))
    purple2=Forbidden_Cmap(range(162,176))
    yellow1=Forbidden_Cmap(range(176,196))
    yellow2=Forbidden_Cmap(range(196,207))
    brown1=Forbidden_Cmap(range(207,228))
    brown2=Forbidden_Cmap(range(228,246))
    brown3=Forbidden_Cmap(range(246,255))
    yellow3=Forbidden_Cmap(range(255,276))
    brown4=Forbidden_Cmap(range(276,293))
    blue1=Forbidden_Cmap(range(293,312))
    blue2=Forbidden_Cmap(range(312,321))
    blue3=Forbidden_Cmap(range(321,333))
    blue4=Forbidden_Cmap(range(333,339))
    grey1=Forbidden_Cmap(range(339,356))
    grey2=Forbidden_Cmap(range(356,385))
    green=green1+green2+green3
    red=red1+red2
    pink=pink1+pink2
    purple=purple1+purple2
    yellow=yellow1+yellow2+yellow3
    brown=brown1+brown2+brown3+brown4
    blue=blue1+blue2+blue3+blue4
    grey=grey1+grey2


    def __init__(self):
        self.color = Forbiddencity
        self.color_pd=pd.DataFrame(self.color).T
    
    def get_cmap(self,color_range,return_cmap=True):
        return Forbidden_Cmap(color_range,return_cmap=return_cmap)
    

    def get_color(self,name):
        return self.color_pd.loc[self.color_pd['name']==name]
    
    def get_color_rgb(self,name):
        return self.color_pd.loc[self.color_pd['name']==name]['color_rgb'].values[0]
    
    def get_cmap_seg(self,colors,N=256):
        from matplotlib.colors import LinearSegmentedColormap
        # Define the colors for the gradient
        colors = np.array(colors) / 255.0
        # Create a colormap from the list of colors
        cmap_name = 'custom_cmap'
        cmap = LinearSegmentedColormap.from_list(cmap_name, colors, N=N)
        return cmap

    # Function to create HTML table with colors
    def visual_color(self, loc_range=(0,385),num_per_row=10):
        df=self.color_pd.iloc[loc_range[0]:loc_range[1]]
        html = '<table style="border-collapse: collapse; width: 100%;">'
        for i in range(0, len(df), num_per_row):
            # Add a row for color names
            html += '<tr>'
            for j in range(i, min(i + num_per_row, len(df))):
                html += f'<td style="border: 1px solid black; padding: 5px;">{df.iloc[j]["name"]}</td>'
            html += '</tr>'
            
            # Add a row for color blocks
            html += '<tr>'
            for j in range(i, min(i + num_per_row, len(df))):
                color = df.iloc[j]["color_html"]
                html += f'<td style="border: 1px solid black; padding: 5px; background-color: {color};"></td>'
            html += '</tr>'
        html += '</table>'
        return html